/*
 * Translate Prolog format CSP to ASP
 * @version 2.1.8a
 * @date 2013-12-18
 */
:- dynamic clause_count/1.

trans(InFile) :-
	open(InFile, read, In),
	repeat,
  	  read(In, X),
	  translate(X),
	X == end_of_file,
	close(In).

translate(end_of_file) :- !.
translate(X) :-
	tr(X, Ss, []),
	!,
	output(Ss).
translate(X) :-
	output([error(X)]).

output([]).
output([S|Ss]) :-
	writeClause(S), write('.'), nl,
	output(Ss).

writeClause(or(X)) :- !,
        write('1 {'), writeClause(X), write('}').
writeClause(not(A)) :- !,
        write('not '), writeClause(A).
writeClause(remdom(A,range(B))) :- !,
        writeClause(remdom(A,B)).
writeClause(remdom(A,range(B,C))) :- !,
        write('remdom('), writeClause(A), write(','),
        writeClause(B), write('..'), writeClause(C), write(')').
writeClause([C]) :- !,
        writeClause(C).
writeClause([C|Cs]) :- !,
        writeClause(C), write(', '), writeClause(Cs).
writeClause(C) :- !,
        writeq(C).

tr([int,V,Domain]) -->
	tr_int(V, Domain).
tr([bool,V]) -->
	tr_bool(V).
tr([relation,R,N,Body]) -->
	tr_relation(R, N, Body).
tr([objective,Target,V]) -->
	[objective(Target,V)].
tr(Clause) -->
	tr_clause(Clause).

/*
 * Integer Variable
 */
tr_int(V, [D|Ds]) -->
        {tr_domain(D, L, U)},
        tr_ints(V, L, U, Ds).
tr_ints(V, LB, UB, []) -->
        [var(V), lb(V, LB), ub(V, UB)].
tr_ints(V, LB, A, [X|Xs]) -->
        {tr_domain(X, L, U)},
        tr_ints(V, LB, U, Xs),
        {A1 is A + 1},
        {L1 is L - 1},
        [remdom(V, range(A1,L1))].

/*
 * Boolean Variable
 */
tr_bool(V) -->
	[bool(V)].

/*
 * Relation
 */
tr_relation(R, N, [Type|Tuples]) -->
	[relation(R, N, Type, Tuples)].

/*
 * Clause
 */
tr_clause(Lit) -->
	tr_literals(Lit).

clause_number(N) :-
	(retract(clause_count(N0)) ; N0 = 0),
	!,
	N is N0 + 1,
	assert(clause_count(N)).

tr_literals(_, []) --> !.
tr_literals(N, [Lit|Lits]) -->
	{tr_literal(Lit, Z)},
	[constraint_elem(N, Z)],
	tr_literals(N, Lits).
tr_literals([]) --> !.
tr_literals(Lit) -->
	{tr_literal(Lit, Z)},
	[Z].

tr_literal(Lit, Lit) :- atomic(Lit), !.
tr_literal([not, Lit], Z) :- !,
	tr_literal(Lit, X),
        Z = not(X).
tr_literal([or, X], Z) :- !,
        tr_literal(X, Z).
tr_literal([or|X], Z) :- !,
        tr_or(X, Xs),
        Z = or(Xs).

tr_literal([wsum, [[A1,X1]], le, B], Z) :- !,
        Z = le0(A1,X1,B).
tr_literal([wsum, [[A1,X1],[A2,X2]], le, B], Z) :- !,
        Z = le(A1,X1,A2,X2,B).
tr_literal([wsum, [[A1,X1],[A2,X2],[A3,X3]], le, B], Z) :- !,
        Z = le(A1,X1,A2,X2,A3,X3,B).
tr_literal([wsum, [[A1,X1]], ge, B], Z) :- !,
        Z = ge0(A1,X1,B).
tr_literal([wsum, [[A1,X1],[A2,X2]], ge, B], Z) :- !,
        Z = ge(A1,X1,A2,X2,B).
tr_literal([wsum, [[A1,X1],[A2,X2],[A3,X3]], ge, B], Z) :- !,
        Z = ge(A1,X1,A2,X2,A3,X3,B).
tr_literal([wsum, AXs, Term, B], Z) :- !,
        append(AXs, B ,AX1),
        flatten(AX1, AX2),
        Z = le(AX2, Term).
tr_literal([alldifferent, Xs], Z) :- !,
	Z = alldifferent(Xs).
tr_literal([count, X, Xs, Comp, Count], Z) :- !,
	Z = count(X, Xs, Comp, Count).
tr_literal([X|Xs], Z) :-
        Z = X, Xs.

tr_domain([], 0, 0).
tr_domain([LB,UB], LB, UB).

tr_or([], []).
tr_or([X|Xs], [Z|Zs]) :- 
        tr_literal(X, Z),
        tr_or(Xs, Zs).
